//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 26 December 1997
// Last Modified: 16 February 1998
// Filename:      ...sig/examples/sigmidi/improv/batonImprov/box.cc
// Syntax:        C++; Radio Baton Improv
//  
// Description: Demonstrates how to create boxes on the radio drum.
//	Notice that the edged of the boxes are very narrow,
//	and the notes can oscillate at the edges.
//

#include "batonImprov.h" 

/*----------------- beginning of improvization algorithms ---------------*/

int boxlevel;    // the height of the four boxes on the surface of the drum

/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   printboxtop();  
   psl(
" Description: Demonstrates how to create boxes    +---------+---------+ ");
   psl(
" on the  radio drum. Notice that the edged of     | note 1  | note 2  | ");
   psl(
" the boxes are very narrow, and the notes can     |         |         | ");
   psl(
" oscillate at the edges.  There are four boxes    | (dial1) | (dial2) | ");
   psl(
" on the drum surface, one for each quadrant.      +---------+---------+ ");
   psl(
" When a stick enters a box, it will generate a    |  note 3 |  note 4 | ");
   psl(
" note.                                            |         |         | ");
   psl(
"    The volume of each box is controlled by       | (dial3) | (dial4) | ");
   psl(
"  each of the four dials as indicated to the      +---------+---------+ ");
   psl(
"  right.  The b14+ raises the height of the boxes, and b15+ lowers      ");
   psl(
"  the height of the box on the surface of the radio drum.               ");
   printboxbottom();
} 

void initialization(void) { 
   boxlevel = 100;
}

void finishup(void) { }


/*-------------------- main loop algorithms -----------------------------*/

int newbox(int batonNumber) {
   // this function returns true if a new box was entered
   switch (batonNumber) {
      case 1:
         // first check to see if the x-axis border was crossed:
         if (midiscale(baton.x1p[0], 0, 1) != 
               midiscale(baton.x1p[-1], 0, 1)) {
            return 1;
         }
         // now check to see if the y-axis boarder was crossed:
         if (midiscale(baton.y1p[0], 0, 1) !=
               midiscale(baton.y1p[-1], 0, 1)) {
            return 1;
         }
         // now check the z-axis
         if (baton.z1p[0] > boxlevel && baton.z1p[-1] <= boxlevel) {
            return 1;
         }
         break;
      case 2:
         // first check to see if the x-axis border was crossed:
         if (midiscale(baton.x2p[0], 0, 1) != 
               midiscale(baton.x2p[-1], 0, 1)) {
            return 1;
         }
         // now check to see if the y-axis boarder was crossed:
         if (midiscale(baton.y2p[0], 0, 1) !=
               midiscale(baton.y2p[-1], 0, 1)) {
            return 1;
         }
         // now check the z-axis
         if (baton.z2p[0] > boxlevel && baton.z2p[-1] <= boxlevel) {
            return 1;
         }
         break;
   } // end of switch

   // no new box 
   return 0;
}

void playbox(int boxnumber) {
   switch (boxnumber) {
      case 11:  
         synth.play(CH_9, GM_LOW_MID_TOM, baton.d1p[0]);
         break;
      case 12:
         synth.play(CH_9, GM_HIGH_MID_TOM, baton.d2p[0]);
         break;
      case 21:
         synth.play(CH_9, GM_LOW_TOM, baton.d3p[0]);
         break;
      case 22:
         synth.play(CH_9, GM_HIGH_TOM, baton.d4p[0]);
         break;
   } // end switch
}

void playboxnote1(void) {
   int boxnumber = 10 * midiscale(baton.x1p[0], 1, 2) +
      midiscale(baton.y1p[0], 1, 2);
   playbox(boxnumber);
}

void playboxnote2(void) {
   int boxnumber = 10 * midiscale(baton.x2p[0], 1, 2) +
      midiscale(baton.y2p[0], 1, 2);
   playbox(boxnumber);
}
      

void mainloopalgorithms(void) { 
   if (newbox(1)) {
      playboxnote1();
   }
   if (newbox(2)) {
      playboxnote2();
   }
}


/*-------------------- triggered algorithms -----------------------------*/

void stick1trig(void) { }

void stick2trig(void) { }

void b14plustrig(void) { 
   boxlevel--;
   if (boxlevel < 0)  boxlevel = 0;
   cout << "boxlevel = " << boxlevel << endl;
}

void b15plustrig(void) { 
   boxlevel++;  
   if (boxlevel > 127)  boxlevel = 127;
   cout << "boxlevel = " << boxlevel << endl;
}

void b14minusuptrig(void) { cout << "b14 pedal released" << endl; }

void b14minusdowntrig(void) { cout << "b14 pedal released" << endl; }

void b15minusuptrig(void) { cout << "b15 pedal released" << endl; }

void b15minusdowntrig(void) { cout << "b15 pedal depressed" << endl; }

void keyboardchar(int key) { charsynth(key); }


/*------------------ end improvization algorithms -----------------------*/



