//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Mar  2 22:32:12 PST 1996
// Last Modified: Thu Feb 12 17:15:23 GMT-0800 1998
// Filename:      ...sig/examples/sigmidi/improv/synthImprov/gliss.cc
// Syntax:        C++; Synth Improv
//

#include "synthImprov.h" 


/*----------------- beginning of improvization algorithms ---------------*/

int octave = 4;              // octave range for computer keyboard notes
int keyboardnote = 0;        // computer keyboard note
MidiMessage noteMessage;     // for reading keyno and velocity (and time)
EventBuffer eventBuffer(2000);  // for future notes
int direction = 1;           // direction of glissandos, 1=up, -1=down
int step = 1;		     // step to take for each note in glissando
int rate = 300;		     // tempo of gliss notes


// user functions not needed by the synthImprov environment:
void playgliss(MidiMessage aMessage);



//////////////////////////////
//
// GlissNoteFunction -- creates glissandos.  To be
//      used with the FunctionNote class; Notes are generated one at a
//      time in the EventBuffer from a FunctionNote.
//

static void GlissNoteFunction(Note* p) {
   Note temp;		// temporary note before placing in buffer

   temp.setOnDur(t_time, p->getOffTime()); // off time holds dur
   temp.play(p->getKey(), p->getVel(), p->getChan());

   p->setKeyno(temp.getKey()+step*direction);
   p->setOnTime(p->getOnTime() + p->getOffTime());  // OffTime stores duration

   eventBuffer.insert(&temp);

   if (p->getKey() > C7 || p->getKey() < A0 ) p->off();
}



/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   cout << endl;
   printboxtop();
   psl(" Gliss -- by Craig Stuart Sapp <craig@ccrma.stanford.edu>, 5 Feb 1998");
   psl("");
   psl(" Description: Creates glissandos when you press a key on the keyboard.");
   psl("    Computer keyboard keys are assigned random attack velocities.");
   printintermediateline();
   psl("   User commands:");
   psl("     \"-\" = decrease step   \"=\" = increase step   \"\\\" = change direction");
   psl("     \"[\" = increase rate   \"]\" = decrease rate ");
   psl("");
   psl("  Computer synthesizer keyboard:");
   psl("      \"0\"-\"9\" = octave number of computer keyboard notes");
   psl("      Notes:           s   d      g    h   j   ");
   psl("                     z   x   c   v   b   n   m  ");
   printboxbottom();
   cout << endl;
} 


void initialization(void) { 
   eventBuffer.setPollRate(10);
}


void finishup(void) { }



/*-------------------- main loop algorithms -----------------------------*/


void mainloopalgorithms(void) { 
   eventBuffer.checkPoll();             // see if any notes need playing

   while (synth.getNoteCount() > 0) {
      noteMessage = synth.extractNote();
      if (noteMessage.p2() != 0) {
         playgliss(noteMessage);
      }
   }
}
      


//////////////////////////////
//
// playgliss -- inserts a gliss functions into the event buffer.
//

void playgliss(MidiMessage aMessage) { 
   Note temp;	// a Temporary Note for copying into eventBuffer

   // setting the fields of the function note, fields are defined in function
   temp.setStatus(FUNCTION_NOTE);
   temp.setTag((char*)&GlissNoteFunction);
   temp.setKeyno(aMessage.p1());
   temp.setVelocity(aMessage.p2());

   // off time of note holds duration:
   temp.setOnOff(t_time, rate);

   eventBuffer.insert(&temp);

   cout << "UP:   StartKey = " << (int)aMessage.p1()
        <<     "\tVelocity = " << (int)aMessage.p2()
        <<     "\tRate = "     << rate
        << endl;
}



/*-------------------- triggered algorithms -----------------------------*/


void keyboardchar(int key) { 
   if (isdigit((char)key)) {
      octave = key - '0';   
      return;
   }
   switch (key) {
      case '-':
         step--;
         if (step < 1)  step = 1;
         cout << "Step = " << step << endl;
         break;
      case '=':
         step++;
         if (step > 12)  step = 12;
         cout << "Step = " << step << endl;
         break;
      case '\\':
         direction *= -1;
         if (direction == 1) {
            cout << "Up" << endl;
         } else {
            cout << "Down" << endl;
         }
         break;
      case '[':
         rate -= 20;
         if (rate < 20)  rate = 20;
         cout << "Rate = " << rate << endl;
         break;
      case ']':
         rate += 20;
         cout << "Rate = " << rate << endl;
         break;
      default:
         octavekeyboard(key);
   }
}



/*------------------ end improvization algorithms -----------------------*/



