//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 18 December 1997
// Last Modified: Mon Jan 26 23:53:05 GMT-0800 1998
// Filename:      ...sig/code/control/MidiOutput/MidiOutput.h
// Syntax:        C++
// $Smake:        cc -g -c %b.cc -I../../../include && rm -f %b.o
//
// Description: The MIDI output interface for MIDI synthesizers/equipment
//

#ifndef _MIDIOUTPUT_H_INCLUDED
#define _MIDIOUTPUT_H_INCLUDED


#include "MidiOutPort.h"
#include "FileIO.h"
#include "Timer.h"


class MidiOutput : public MidiOutPort {
   public:
                MidiOutput     (void);
                MidiOutput     (int aPort, int autoOpen = 1);
               ~MidiOutput     ();

      // Basic user MIDI output commands:
      int       cont           (int channel, int controller, int data);
      int       off            (int channel, int keynum, int releaseVelocity);
      int       pc             (int channel, int timbre);
      int       play           (int channel, int keynum, int velocity);
      int       pw             (int channel, int mostByte, int leastByte);
      int       pw             (int channel, int tuningData);
      int       pw             (int channel, double tuningData);
      void      recordStart    (char *filename, int format);
      void      recordStop     (void);
      void      reset          (void);
      int       send           (int command, int p1, int p2);
      int       send           (int command, int p1);
      int       send           (int command);
      void      silence        (int aChannel = -1);
      void      sustain        (int channel, int status);

      //int       sysex          (char* data, int length);

   protected:
      int       outputRecordQ;     // boolean for recording
      int       outputRecordType;  // what form to record MIDI data in
      int       lastFlushTime;     // for recording midi data
      FileIO    outputRecordFile;  // file for recording midi data
      static Timer timer;          // for recording midi data

      void      writeOutputAscii   (int channel, int p1, int p2);
      void      writeOutputBinary  (int channel, int p1, int p2); 
      void      writeOutputMidifile(int channel, int p1, int p2);

};



#endif // _MIDIOUTPUT_H_INCLUDED



// Brief description of MidiOutput public member functions:
//
// send:    sends a MIDI command to the MIDI output with up to two parameters
// play:    sends a playnote command to the MIDI output
// pc:      Patch Change.  changes the timbre (instrument) on the given channel
// cont:    sends a CONTinuous CONTroller MIDI command
// sysex:   sends a system exclusive command to the MIDI output
//


